*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
* cyclic.f - circular averages
*  by Andy Allinger, 2013, released to the public domain
*  send bug reports to:  lexicographer@pseudolog.net
*       TRIGAV:  vector sum average
*       CYMEAN:  cyclic mean by a method due to David Radcliffe, 2005
*       CYMED:  cyclic median by a new variation of Radcliffe's algorithm
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*  TRIGAV - trigonometric-based average for cyclic data
*   by Andy Allinger, 2013, released to the public domain
*     D [n]       Real     In     data array
*     n           Integer  In     length of D
*     t           Real     In     period of modulo division
*     ave         Real     Out    average phase
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      SUBROUTINE TRIGAV (D, n, t, ave)
       IMPLICIT NONE
       INTEGER n
       REAL D, t, ave
       DIMENSION D(n)
*          locals       
       INTEGER i
       REAL x, y, h, twopi, fact
       PARAMETER (twopi = 6.2831853072)
       
*             begin
       fact = twopi / t
       x = 0. ; y = 0.
       DO i = 1, n
         h = fact * MOD(D(i), t) 
         x = x + COS(h)
         y = y + SIN(h)
       END DO
       h = ATAN2(y,x)
       ave = h / fact
      END ! of TRIGAV


*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
* CYMEAN - cyclical mean
*   by Andy Allinger, 2013, released to the public domain
* origin:  David Radcliffe, 2005 (or earlier)
*
*      He proposed the algorithm:
* 1. Find the average and standard deviation of the given numbers.
* 2. Increase the smallest number by 360.
* 3. Repeat steps 1 and 2 until all numbers are greater than 360.
* 4. Choose the average that yields the smallest standard deviation.
* 5. If the average is greater than 360 then subtract 360.
*
* Notice that unlike the linear mean, the cyclical mean does not
* always have a distinct value.  Instead, the array ave[] contains in
* its beginning entries the various averages, and the integer ties denotes
* how many averages were found.  
* WARNING:  X is returned shifted and sorted
*
*     X [n]       Real     In     data array
*     n           Integer  In     length of X
*     t           Real     In     period 
*     AVE [n]     Real     Out    average
*     ties        Integer  Out    how many values in ave[]
*     dev         Real     Out    standard deviation
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      SUBROUTINE CYMEAN (X, n, t, AVE, ties, dev)
       IMPLICIT NONE
       INTEGER n, ties
       REAL X, t, AVE, dev
       DIMENSION X(n), AVE(n)
       
       INTEGER i
       REAL 
     &      mu,             ! arithmetic mean
     &      r               ! temporary scalar X[i]
       DOUBLE PRECISION 
!     &                  dmachine,  ! function when EPSILON() not available
     &                  nn,        ! double precision of n
     &                  s, s2,     ! sum, sum of squares
     &                  v, vmin,   ! n @ variance, minimal n @ variance
     &                  tt, t2,    ! period, period squared
     &                  tol,       ! a small number
     &                  rr         ! double precision of r
       
*        begin
       IF (n .EQ. 1) THEN 
         ties = 1
         AVE(1) = X(1)
         dev = 0.
         RETURN
       END IF
 
       nn = DBLE(n) ; tt = DBLE(t) 
       tol = SQRT(tt * EPSILON(tol))
!       tol = SQRT(tt * dmachine(1))
       t2 = tt **2
       s = 0. ; s2 = 0. ; mu = 0.
              
*        move all data to the range 0...t, sort
       DO i = 1, n
         r = X(i)
         mu = mu + r
         r = MOD(r, t)
         IF (r < 0.) r = r + t
         rr = DBLE(r)
         s = s + rr ; s2 = s2 + rr **2        ! accumulate sums
         X(i) = r
       END DO
       mu = mu / SNGL(nn)
       CALL SSORT (X, n)
       
*           find phase of seam of lowest deviation
       vmin = HUGE(tt)
!       vmin = dmachine(3) 
       ties = 0
       DO i = 1, n
         v = s2 - s **2 / nn              ! n @ variance
         IF (vmin - v > tol) THEN          ! new low deviation
           ties = 0
           vmin = v
         END IF
         IF (ABS(v - vmin) < tol) THEN  ! tie for new low deviation           
           ties = ties + 1
           AVE(ties) = SNGL(s / nn)
         END IF
         
*           apply update formula to sum and sum of squares
         s = s + tt
         s2 = s2 + 2D0 * DBLE(X(i)) * tt + t2
       END DO
       
*           preserve cycle information
       DO i = 1, ties
         r = AVE(i)
         IF (r .GE. t) r = r - t      ! subtract if out of 0...t
         r = MOD(r - mu, t)   ! shortest path from linear to cyclical average
         IF (r < -t/2.) THEN
           r = r + t
         ELSE IF (r > t/2.) THEN
           r = r - t
         END IF
         AVE(i) = mu + r
       END DO
       dev = SNGL(SQRT(vmin / nn))                ! (n@variance / n)^(1/2)
       
      END ! of CYMEAN


*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
* CYMED - cyclical median, a variation of Radcliffe's algorithm
*   by Andy Allinger, 2013, released to the public domain
* The cyclical median does not always have a distinct value.
*   WARNING:  X is returned shifted and sorted
*
*     X [n]       Real     In     data array
*     n           Integer  In     length of X
*     t           Real     In     period
*     AVE [n]     Real     Out    average
*     ties        Integer  Out    how many values in AVE[]
*     dev         Real     Out    deviation
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      SUBROUTINE CYMED (X, n, t, AVE, ties, dev)
       IMPLICIT NONE
       INTEGER n, ties
       REAL X, t, AVE, dev
       DIMENSION X(n), AVE(n)
       
       INTEGER i,         ! count
     &         j0, j1     ! positions where to find current estimate of median
       REAL
     &      mu,             ! linear median on input X[]
     &      r               ! temporary copy of X
       DOUBLE PRECISION 
!     &                  dmachine,  ! function when EPSILON() not available
     &                  s1, smin,  ! sum of deviations, minimum sum deviation
     &                  tt,        ! period
     &                  tol        ! a small number
       LOGICAL odd              ! even or odd n
       
*        begin
       IF (n .EQ. 1) THEN
         ties = 1
         AVE(1) = X(1)
         dev = 0.         
         RETURN
       END IF
       j1 = n / 2 + 1
       odd = MOD(n, 2) .EQ. 1              ! is n an odd number?
       tt = DBLE(t)
       tol = tt * EPSILON(tol)
!       tol = tt * dmachine(1)
       CALL median(X, n, mu)
              
*        move all data to the range 0...t, sort
       DO i = 1, n
         r = MOD(X(i), t)
         IF (r < 0.) r = r + t
         X(i) = r
       END DO
       CALL SSORT (X, n)

*            initial value of cyclical median
       IF (odd) THEN
         r = X(j1)
       ELSE
         j0 = j1 - 1
         r = (X(j0) + X(j1)) / 2.     ! split the difference
       END IF

*         sum of absolute deviations
       s1 = 0. 
       DO i = 1, n
         s1 = s1 + ABS(X(i) - r)
       END DO
       
*           find phase of seam of lowest deviation
       smin = HUGE(tt)
!       smin = dmachine(3)
       ties = 0
       DO i = 1, n
         IF (smin - s1 > tol) THEN          ! new low deviation
           ties = 0
           smin = s1
         END IF
         IF (ABS(s1 - smin) < tol) THEN  ! tie for new low deviation           
           ties = ties + 1
           IF (odd) THEN
             AVE(ties) = X(j1)
           ELSE              ! two points required for even case 
             j0 = j1 - 1 ; IF (j0 .EQ. 0) j0 = n   ! j0 is the prior point 
             AVE(ties) = (X(j0) + X(j1)) / 2. 
           END IF 
         END IF
         
*           apply update formula        
         r = X(i)
         IF (odd) THEN
           j0 = j1 + 1 ; IF (j0 > n) j0 = 1        ! new and current medians
           s1 = s1 + 2. * r + tt - DBLE(X(j0) + X(j1))   ! j0 is the new median
         ELSE
           s1 = s1 + 2. * r + tt - 2. * X(j1)
         END IF
         X(i) = r + t               ! advance seam
         j1 = j1 + 1 ; IF (j1 > n) j1 = 1         
       END DO

*         preserve cycle information
       DO i = 1, ties       
         r = AVE(i)
         IF (r .GE. t) r = r - t
         r = MOD(r - mu, t)     ! shortest path from linear to cyclical average
         IF (r < -t/2.) THEN
           r = r + t
         ELSE IF (r > t/2.) THEN
           r = r - t
         END IF
         AVE(i) = mu + r
       END DO
       
       dev = SNGL(smin) / FLOAT(n)                     ! deviation
      END ! of CYMED

*%%%%%%%%%%%%%%%%%%%%%%% End of file cyclic.f %%%%%%%%%%%%%%%%%%%%%%%%%%
