*-----------------------------------------------------------------------
* SSORT - sort single precision numbers
*-----------------------------------------------------------------------
*   Sort an array in increasing order.  A slightly modified QUICKSORT 
* algorithm is used.  From the SLATEC library, public domain. 
*
*      By  R.E. Jones, (SNLA) 
*      and J. A. Wisniewski, (SNLA)
*
*   Description of arguments
*      X[]     array of values to be sorted
*      N       number of values in array X to be sorted
*
*  REFERENCES  R. C. Singleton, Algorithm 347, An efficient algorithm
*                 for sorting with minimal storage, Communications of
*                 the ACM, 12, 3 (1969), pp. 185-187.
*
*  REVISION HISTORY  (YYMMDD)
*   761101  Date written
*   761118  Modified to use the Singleton quicksort algorithm.  (JAW)
*   890531  Changed all specific intrinsics to generic.  (WRB)
*   890831  Modified array declarations.  (WRB)
*   891009  Removed unreferenced statement labels.  (WRB)
*   891024  Changed category.  (WRB)
*   891024  REVISION DATE from Version 3.2
*   891214  Prologue converted to Version 4.0 format.  (BAB)
*   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
*   901012  Declared all variables; changed X,Y to SX,SY. (M. McClain)
*   920501  Reformatted the REFERENCES section.  (DWL, WRB)
*   920519  Clarified error messages.  (DWL)
*   920801  Declarations section rebuilt and code restructured to use
*           IF-THEN-ELSE-ENDIF.  (RWC, WRB)
*   110517  removed call to XERMSG, simplified (AJA)
*-----------------------------------------------------------------------
      SUBROUTINE SSORT (X, N)

*     .. Scalar Arguments ..
      INTEGER  N
*     .. Array Arguments ..
      REAL X(N)
*     .. Local Scalars ..
      REAL R, T, TT
      INTEGER I, IJ, J, K, L, M, NN
*     .. Local Arrays ..
      INTEGER IL(32), IU(32)
*-----------------------------------------------------------------------
      NN = N
      IF (NN < 1) THEN
          PRINT *, 'The number of values to be sorted is not positive.'
         RETURN
      ENDIF

*     Sort X only
      M = 1
      I = 1
      J = NN
      R = 0.375E0

   20 IF (I .EQ. J) GO TO 60
      IF (R .LE. 0.5898437E0) THEN
         R = R+3.90625E-2
      ELSE
         R = R-0.21875E0
      ENDIF

   30 K = I

*     Select a central element of the array and save it in location T
      IJ = I + INT(FLOAT(J-I)*R)
      T = X(IJ)

*     If first element of array is greater than T, interchange with T
      IF (X(I) > T) THEN
         X(IJ) = X(I)
         X(I) = T
         T = X(IJ)
      ENDIF
      L = J

*     If last element of array is less than than T, interchange with T
      IF (X(J) < T) THEN
         X(IJ) = X(J)
         X(J) = T
         T = X(IJ)

*        If first element of array is greater than T, interchange with T
         IF (X(I) > T) THEN
            X(IJ) = X(I)
            X(I) = T
            T = X(IJ)
         ENDIF
      ENDIF

*     Find an element in the second half of the array which is smaller than T
   40 L = L-1
      IF (X(L) > T) GO TO 40

*     Find an element in the first half of the array which is greater than T
   50 K = K+1
      IF (X(K) < T) GO TO 50

*     Interchange these elements
      IF (K .LE. L) THEN
         TT = X(L)
         X(L) = X(K)
         X(K) = TT
         GO TO 40
      ENDIF

*     Save upper and lower subscripts of the array yet to be sorted
      IF (L-I > J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 70

*     Begin again on another portion of the unsorted array
   60 M = M-1
      IF (M .EQ. 0) GO TO 200
      I = IL(M)
      J = IU(M)

   70 IF (J-I .GE. 1) GO TO 30
      IF (I .EQ. 1) GO TO 20
      I = I-1

   80 I = I+1
      IF (I .EQ. J) GO TO 60
      T = X(I+1)
      IF (X(I) .LE. T) GO TO 80
      K = I

   90 X(K+1) = X(K)
      K = K-1
      IF (T < X(K)) GO TO 90
      X(K+1) = T
      GO TO 80

  200 END ! of SSORT
  
*************************** end of file ssort.f ************************
